/*********************************************************
  Example "C" program showing use of Boolean Gadgets.         

  This was compiled using Manx, Aztec "C", AmigaDos V1.2      
  The code depends on V1.2 functions to operate correctly.	
  It is intended to be run from CLI.				
 								
  Note: All Image data must reside in CHIP memory ( lower     
  512K). If using Lattice, run ATOM on the compiled output    
  before linking. If using Manx, use the +Cdb switch on 
  the linker.							
								
  Copyright (C) 1987 H. Maybeck Tolly, TollySoft		
  This program is in the public domain and may be 
  distributed free of charge.                                             
                                                              
***********************************************************/

#include <exec/types.h>
#include <exec/exec.h>
#include <intuition/intuition.h>
#include <graphics/gfxbase.h>
#include <functions.h>

struct IntuitionBase *IntuitionBase =0L;
struct GfxBase *GfxBase = 0L;
struct Window *ControlWindow = NULL;
struct IntuiMessage *MyIntuiMessage;


/********************************************/
/* Border for toggle gadget (BoolGadget[1]) */
/********************************************/

WORD BoolVectors[] = {0, 0,  51, 0,  51, 41,  0, 41,  0,0};
struct Border BoolBorder = {
   -1, -1,            /* initial offsets, gadget relative*/
   3, 2, JAM1,        /* pens (fore, back) and drawmode */
   5,                 /* number of vectors */
   (SHORT *)BoolVectors, /* pointer to the actual array */
			 /* of vectors */
   NULL                    /* no next border */
};

/* All image data must reside in CHIP memory. */

/*******************************************************/
/* Two Images for hit Gadget with altername images     */
/* (BoolGadget[2])                                     */
/*******************************************************/

/* Color 3 'X', 2 planes */

UWORD imageone[] = {
	0xFFF0, 0x0000, 0x0000, 0x0FFF,
	0x000F, 0xFF00,	0x00FF, 0xF000,
	0x0000, 0xFFF0,	0x0FFF, 0x0000,
	0x0000, 0x00FF,	0xFF00, 0x0000,
	0x0000, 0x000F,	0xF000, 0x0000,
	0x0000, 0x000F, 0xF000, 0x0000,
	0x0000, 0x00FF,	0xFF00, 0x0000,
	0x0000, 0xFFF0,	0x0FFF, 0x0000,
	0x000F, 0xFF00,	0x00FF, 0xF000,
	0xFFF0, 0x0000,	0x0000, 0x0FFF,

	0xFFF0, 0x0000, 0x0000, 0x0FFF,
	0x000F, 0xFF00,	0x00FF, 0xF000,
	0x0000, 0xFFF0,	0x0FFF, 0x0000,
	0x0000, 0x00FF,	0xFF00, 0x0000,
	0x0000, 0x000F,	0xF000, 0x0000,
	0x0000, 0x000F, 0xF000, 0x0000,
	0x0000, 0x00FF,	0xFF00, 0x0000,
	0x0000, 0xFFF0,	0x0FFF, 0x0000,
	0x000F, 0xFF00,	0x00FF, 0xF000,
	0xFFF0, 0x0000,	0x0000, 0x0FFF
	};

/* Color 2  '+', 2 planes */

UWORD imagetwo[] = {
	0x0000, 0x0000, 0x0000, 0x0000,
	0x0000, 0x0000, 0x0000, 0x0000,
	0x0000, 0x0000, 0x0000, 0x0000,
	0x0000, 0x0000, 0x0000, 0x0000,
	0x0000, 0x0000, 0x0000, 0x0000,
	0x0000, 0x0000, 0x0000, 0x0000,
	0x0000, 0x0000, 0x0000, 0x0000,
	0x0000, 0x0000, 0x0000, 0x0000,
	0x0000, 0x0000, 0x0000, 0x0000,
	0x0000, 0x0000, 0x0000, 0x0000,

	0x0000, 0x0FFF,	0xFFF0, 0x0000,
	0x0000, 0x0FFF,	0xFFF0, 0x0000,
	0x0000, 0x0FFF,	0xFFF0, 0x0000,
	0xFFFF, 0xFFFF,	0xFFFF, 0xFFFF,
	0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF,
	0xFFFF, 0xFFFF, 0xFFFF, 0xFFFF,
	0xFFFF, 0xFFFF,	0xFFFF, 0xFFFF,
	0x0000, 0x0FFF,	0xFFF0, 0x0000,
	0x0000, 0x0FFF,	0xFFF0, 0x0000,
	0x0000, 0x0FFF,	0xFFF0, 0x0000
	};

struct Image BoolImage = {
 0,0,64,10,2,
 imageone,
 0x03, 0x0, NULL
 };

struct Image BoolHImage = {
 0,0,64,10,2,
 imagetwo,
 0x03, 0x0, NULL
 };

/*******************************************/
/* Image for masked gadget (BoolGadget[3]) */
/*******************************************/

USHORT gadget_imageData[] =
{

0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x000F,0xF000,0x0000,
0x0000,0x00FF,0xFF00,0x0000,
0x0000,0x0FFF,0xFFF0,0x0000,
0x0000,0xFFFF,0xFFFF,0x0000,
0x0000,0xFFFF,0xFFFF,0x0000,
0x000F,0xFFFF,0xFFFF,0xF000,
0x000F,0xFFFF,0xFFFF,0xF000,
0x000F,0xFFFF,0xFFFF,0xF000,
0x00FF,0xFFFF,0xFFFF,0xFF00,
0x00FF,0xFFFF,0xFFFF,0xFF00,
0x00FF,0xFFFF,0xFFFF,0xFF00,
0x0FFF,0xFFFF,0xFFFF,0xFFF0,
0x0FFF,0xFFFF,0xFFFF,0xFFF0,
0x00FF,0xFFFF,0xFFFF,0xFF00,
0x00FF,0xFFFF,0xFFFF,0xFF00,
0x00FF,0xFFFF,0xFFFF,0xFF00,
0x000F,0xFFFF,0xFFFF,0xF000,
0x000F,0xFFFF,0xFFFF,0xF000,
0x000F,0xFFFF,0xFFFF,0xF000,
0x0000,0xFFFF,0xFFFF,0x0000,
0x0000,0xFFFF,0xFFFF,0x0000,
0x0000,0x0FFF,0xFFF0,0x0000,
0x0000,0x00FF,0xFF00,0x0000,
0x0000,0x000F,0xF000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,

0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x000F,0xF000,0x0000,
0x0000,0x00FF,0xFF00,0x0000,
0x0000,0x00FF,0xFF00,0x0000,
0x0000,0x0FFF,0xFFF0,0x0000,
0x0000,0x0FFF,0xFFF0,0x0000,
0x0000,0x0FFF,0xFFF0,0x0000,
0x0000,0x00FF,0xFF00,0x0000,
0x0000,0x00FF,0xFF00,0x0000,
0x0000,0x000F,0xF000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000

};

USHORT bool_mask[] =
{

0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x000F,0xF000,0x0000,
0x0000,0x00FF,0xFF00,0x0000,
0x0000,0x00FF,0xFF00,0x0000,
0x0000,0x0FFF,0xFFF0,0x0000,
0x0000,0x0FFF,0xFFF0,0x0000,
0x0000,0x0FFF,0xFFF0,0x0000,
0x0000,0x00FF,0xFF00,0x0000,
0x0000,0x00FF,0xFF00,0x0000,
0x0000,0x000F,0xF000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000,
0x0000,0x0000,0x0000,0x0000
};

struct Image BoolMaskImage = {
    0,0,               /* LeftEdge, TopEdge */
    64,30,2,           /* Width, Height, Depth */
    &gadget_imageData[0],
    0x03,0x00,         /* PlanePick, PlaneOnOff */
    NULL,              /* Pointer to next Image */
};

struct BoolInfo BoolInfo = {
    BOOLMASK,	     /* This is currently the only */
		     /* flag for boolean gadgets */
    &bool_mask[0],   /* Pointer to the mask */
    NULL
};
	
   struct IntuiText IntuiStr_blank = 
	{2,3,JAM2,5,5,NULL,(UBYTE *)"   ",NULL};

   struct IntuiText IntuiStr_yes = 
	{2,3,JAM2,5,5,NULL,(UBYTE *)"YES",NULL};

   struct IntuiText IntuiStr_no = 
	{2,3,JAM2,5,5,NULL,(UBYTE *)"NO",NULL};

/*******************************************************/
/* Gadget structures for three example gadgets         */
/*******************************************************/
 
struct Gadget BoolGadget[5] = {
{

/* Hit gadget with Border */
   NULL,             /* pointer to Next Gadget */
   10,20,50,40,      /* (Left Top Width Height) Hit Box*/
   GADGHCOMP ,       /* Flags */
   GADGIMMEDIATE | RELVERIFY ,   /* Activation flags */
   BOOLGADGET,        /* Type */
   (APTR)&BoolBorder, /* pointer to Border Image */
   NULL,              /* no pointer to SelectRender */
   &IntuiStr_yes,     /* pointer to GadgetText */
   0,                 /* no MutualExclude */
   NULL,              /* pointer to SpecialInfo */
   0,                 /* no ID */
   NULL               /* no pointer to special data */
},

/* Alternate Image gadget */
   {&BoolGadget[0],110,20,64,10,GADGHIMAGE | GADGIMAGE ,
      GADGIMMEDIATE | RELVERIFY | TOGGLESELECT,BOOLGADGET,
      (APTR)&BoolImage,(APTR)&BoolHImage,&IntuiStr_yes,
      0,NULL,0,NULL},

/* This gadget has mask. It must have SpecialInfo point */
/* to a BoolInfo, and have the BOOLEXTEND flag set in   */
/* its activation flags.                                */

   {&BoolGadget[1],220,20,64,30,GADGHCOMP | GADGIMAGE  ,
     GADGIMMEDIATE | RELVERIFY | TOGGLESELECT | BOOLEXTEND,
     BOOLGADGET,(APTR)&BoolMaskImage,NULL,NULL,0,
     (APTR)&BoolInfo,0,NULL},

/* Mutual Exclude pair */
   {&BoolGadget[2],10,120,64,10,GADGHCOMP | GADGIMAGE | 
	SELECTED,GADGIMMEDIATE, BOOLGADGET,(APTR)&BoolImage,
	NULL,NULL,0,NULL,0,NULL},
   {&BoolGadget[3],110,120,64,10,GADGHCOMP | GADGIMAGE  ,
	GADGIMMEDIATE, BOOLGADGET ,(APTR)&BoolHImage,NULL,
	NULL,0,NULL,0,NULL}

};

struct NewWindow NewControlWindow = {
   20, 20,               /* start LeftEdge, TopEdge */
   320, 160,             /* start Width, Height */
   2, 3,                 /* DetailPen, BlockPen */
		         /* IDCMP FLAGS */
   GADGETUP | GADGETDOWN | CLOSEWINDOW | MENUPICK,    
		         /* Flags */
   WINDOWDRAG | WINDOWDEPTH | WINDOWCLOSE | ACTIVATE,   
   &BoolGadget[4],       /* Pointer to FirstGadget */
   NULL,                 /* no pointer to first CheckMark*/
   (UBYTE *)"Boolean Gadgets",/* Title */
   NULL,                 /* no Pointer to Screen */
   NULL,                 /* no Pointer to BitMap */
   20, 20,               /* Min size (no size allowed) */
   320, 160,             /* Max size (no size allowed) */
   WBENCHSCREEN          /* Type of screen */
};

/********************************************************/
/* Menu declarations                                    */
/********************************************************/

   struct IntuiText ghost_text = {
/* Frontpen, Backpen, Draw Mode   */
        2,1,JAM2,                      
/* Left and Top offsets, Font     */
        5,1,NULL,                      
/* Text to display,next IntuiText */
        (UBYTE *)"GHOST",NULL  } ;     

   struct MenuItem ghost_item = {
/* Next item, left,top,width,height */
        NULL,0,5,140,10,            
        HIGHCOMP | ITEMENABLED | ITEMTEXT | COMMSEQ,
/* MutualExclude, ItemFill (text)    */
        NULL, (APTR) &ghost_text,  
/* SelectFill, Command, Subitem      */
        NULL, 'G', NULL } ;        

   struct Menu gadget_menu = {
/* Next menu, left,top,width,height */
        NULL, 0, 0, 60, 10,        
/* Flags, text                      */
        MENUENABLED, "Gadgets",    
/* Pointer to first menu item       */
        &ghost_item } ;            

/****************************************************/
/* Main program                                     */
/****************************************************/

main()
{
   struct MenuItem *ItemAddress();
   ULONG Signals, MIClass, MICode, itemnum;        
   APTR MIAddress;
   LONG gad_pos, real_pos;

/* Open libraries */

   if (!(IntuitionBase = (struct IntuitionBase *)
    OpenLibrary("intuition.library", 
		(LONG)LIBRARY_VERSION)))
   {
      printf("Can't open the intuition library\n");
      MyCleanup();
      exit(FALSE);
   }

   if (!(GfxBase = (struct GfxBase *)
    OpenLibrary("graphics.library", 
		(LONG)LIBRARY_VERSION)))
   {
      printf("Can't open the graphics library\n");
      MyCleanup();
      exit(FALSE);
   }

/* Open window in which to display Boolean Gadgets. */

      if (!(ControlWindow = 
	(struct Window *)OpenWindow(&NewControlWindow)))
      {
         printf("Couldn't open the control window.\n");
         MyCleanup();
         exit(FALSE);
      }

   SetMenuStrip(ControlWindow, &gadget_menu);

/* Loop forever until user clicks Close Gadget on window.*/

   for (;;) {  /* wait for a signal and process it */

      Signals = Wait(1L << 
	ControlWindow->UserPort->mp_SigBit);

         /* Process the Intuition message. */
         while (MyIntuiMessage=(struct IntuiMessage *)
                      GetMsg(ControlWindow->UserPort)) 
         {
         /* Get all the needed info and reply to message. */
         MIClass = MyIntuiMessage->Class;
         MICode = MyIntuiMessage->Code;
         MIAddress = MyIntuiMessage->IAddress;
         ReplyMsg(MyIntuiMessage);
         /* Determine what the message was. */

           switch (MIClass) 
	   {
             case MENUPICK:

             while (MICode != MENUNULL)
             {
               itemnum = ITEMNUM(MICode);
               switch(itemnum)
               {
/* User chose to ghost Masked Gadget. */
                  case 0:

/* Remove the Gadget from Intuition's control */
/* before we change any values.		       */
                  gad_pos = RemoveGList(ControlWindow,
			&BoolGadget[2],1L);

/* Ghost/Unghost the Masked Gadget. */
                  BoolGadget[2].Flags ^= GADGDISABLED;

/* Return the Gadget to Intuition's control. */
		  real_pos = AddGList(ControlWindow, 
			&BoolGadget[2], gad_pos, 1L, 
			(LONG)NULL);

/* Refresh the display of the Gadget. */
                  RefreshGList(&BoolGadget[2], 
			ControlWindow, (LONG)NULL,1L);

                  break;

               } /* switch */

/* Get next Code if user extend-selected menu items. */
               MICode = (ItemAddress(&gadget_menu,MICode))
				->NextSelect;

             } /* while */
             break;

	     case GADGETUP:
	
/* Check for GADGETUP on hit Gadget with IntuiText. */
             if (MIAddress == &BoolGadget[0])
	     {

/* Write out a blank string to cover up old characters. */

/* Remove the Gadget from Intuition's control */
/* before we change any values.		       */
	     gad_pos = RemoveGList(ControlWindow,
		&BoolGadget[0],1L);

	     BoolGadget[0].GadgetText = &IntuiStr_blank;

/* Return the Gadget to Intuition's control. */
	     real_pos = AddGList(ControlWindow, 
		&BoolGadget[0],	gad_pos, 1L, (LONG)NULL);

/* Refresh the display of the Gadget. */
             RefreshGList(&BoolGadget[0], ControlWindow, 
		(LONG)NULL,1L);

/* Now write out the new string. */

/* Remove the Gadget from Intuition's control */
/* before we change any values.		       */
	     gad_pos = RemoveGList(ControlWindow,
		&BoolGadget[0],1L);

	     BoolGadget[0].GadgetText = &IntuiStr_no;

/* Return the Gadget to Intuition's control. */
             real_pos = AddGList(ControlWindow, 
		&BoolGadget[0],gad_pos, 1L, (LONG)NULL);

/* Refresh the display of the Gadget. */
             RefreshGList(&BoolGadget[0], ControlWindow, 
		(LONG)NULL,1L);
	     }

 	     break;

	     case GADGETDOWN:

/* Check for GADGETDOWN on Mutual Exclude Gadgets. */
	     if (MIAddress == &BoolGadget[3] || MIAddress 
			== &BoolGadget[4])
	     {
	     gad_pos = RemoveGList(ControlWindow, 
		&BoolGadget[4], 2L);

	     BoolGadget[3].Flags ^= SELECTED;
	     BoolGadget[4].Flags ^= SELECTED;

	     AddGList(ControlWindow, &BoolGadget[4], 
		gad_pos, 2L, NULL); 
             RefreshGList(&BoolGadget[4], ControlWindow, 
		(LONG)NULL,2L);
	     }

	     break;

/* User clicked close Gadget. */
             case CLOSEWINDOW:       

/* Reply to any outstanding messages. */
             while (MyIntuiMessage =(struct IntuiMessage *)
		 GetMsg(ControlWindow->UserPort))
                    ReplyMsg(MyIntuiMessage);
             MyCleanup();
             exit(TRUE);
             break;

           }  /* switch */
         } /* while */
   }  /* for */
}  /* main */

MyCleanup()
{
   if (ControlWindow) CloseWindow(ControlWindow);
   if (GfxBase) CloseLibrary(GfxBase);
   if (IntuitionBase) CloseLibrary(IntuitionBase);
}

